function getPosition(evt) {
    evt = (evt) ?  evt : ((event) ? event : null);
    var left = 0;
    var top = 0;

    if (evt.pageX) {
        left = evt.pageX;
        top  = evt.pageY;
    } else {
        left = evt.clientX + document.documentElement.scrollLeft;
        top  = evt.clientY + document.documentElement.scrollTop;
    }
 
    return {x : left, y : top}; 
}

Ryoku.prototype.drawDot = function(x,y) {
    var dot = document.createElement("span");
    dot.style.left = x + "px";
    dot.style.top =  y + "px";
    dot.className = "dot";
    this.canvas.appendChild(dot);
    this.points[this.point_num++] = [x-this.offsetLeft, y-this.offsetTop];
}

Ryoku.prototype.drawLine = function(x1,y1,x2,y2) {
    if (x1 == x2 && y1 == y2) return;

    var x_move = x2 - x1;
    var y_move = y2 - y1;
    var x_diff = x_move < 0 ? 1 : -1;
    var y_diff = y_move < 0 ? 1 : -1;

    if (Math.abs(x_move) >= Math.abs(y_move)){
        for (var i = x_move; i != 0; i += x_diff) {
            this.drawDot(x2 - i, y2 - Math.round(y_move * i / x_move));
        }
    } else {
        for (var i = y_move; i != 0; i += y_diff) {
            this.drawDot(x2 - Math.round(x_move * i / y_move), y2 - i);
        }
    }
}

Ryoku.prototype.trace = function (event) {
    var pos = getPosition(event);
    if (this.prev_pos) {
        this.drawLine(this.prev_pos.x, this.prev_pos.y, pos.x, pos.y);
    }
    this.prev_pos = pos;
}

function createXmlHttp() {
    try {
        xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
    } catch (e) {
        try {
            xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
        } catch (e) {
            xmlhttp = false;
        }
    }
    if (!xmlhttp && typeof XMLHttpRequest != 'undefined') {
        xmlhttp = new XMLHttpRequest();
    }
    return xmlhttp;
}

Ryoku.prototype.request = function(msg) {
    var xmlhttp = createXmlHttp();
    var self = this;
    xmlhttp.open("GET", msg, true);
    xmlhttp.onreadystatechange=function() {
        if (xmlhttp.readyState == 4 && xmlhttp.status == 200) {
            this.res = xmlhttp.responseText;
            self.canvas.style.backgroundImage = 'url(' + this.res + ')';
            self.canvas.innerHTML = "";
        }
    }
    xmlhttp.send(null);
}

Ryoku.prototype.mouseup = function(event) {
    if (!this.active) return;

    this.trace(event);

    this.active = false;
    var msg = "";
    for (var i = 0; i < this.point_num; i++) {
        msg += this.points[i][0] + "," + this.points[i][1] + ",";
    }

    this.request("write.cgi?"+msg);

    this.reset();
}

Ryoku.prototype.mousemove = function(event) {
    if (!this.active) return;
    this.trace(event);
}

Ryoku.prototype.mousedown = function(event) {
    this.active = true;
    this.trace(event);
}

Ryoku.prototype.reset = function() {
    this.points = [];
    this.point_num = 0;
    this.prev_pos = null;
}

Ryoku.prototype.reload = function() {
    var xmlhttp = createXmlHttp();
    var self = this;
    xmlhttp.open("GET", "serv.cgi", true);
    xmlhttp.onreadystatechange=function() {
        if (xmlhttp.readyState == 4) {
            if (xmlhttp.status == 200) {
                this.res = xmlhttp.responseText;
                self.canvas.style.backgroundImage = 'url(' + this.res + ')';
                self.canvas.innerHTML = "";
            }
            self.reload();
        }
    }
    xmlhttp.send(null);
}

function Ryoku() {
    var canvas = document.getElementById("canvas");
    this.canvas = canvas;

    var self = this;
    canvas.onmouseup    = function(event) { self.mouseup(event); }
//    canvas.onmouseleave = function(event) { self.mouseup(event); }
    canvas.onmousedown  = function(event) { self.mousedown(event); }
    canvas.onmousemove  = function(event) { self.mousemove(event); }

    var left = 0;
    var top = 0;
    for (var o = canvas; o ; o = o.offsetParent) {
        left += (o.offsetLeft - o.scrollLeft);
        top  += (o.offsetTop - o.scrollTop);
    }
    this.offsetLeft = left + 11;
    this.offsetTop  = top + 11;

    this.reset();

    this.request("write.cgi");

    this.reload()
}

var ryoku;
onload = function () {
    ryoku = new Ryoku();
}

Ryoku.prototype.undo = function() {
    this.request("write.cgi?undo");
}

function ryoku_undo() {
    ryoku.undo();
    return false;
}

Ryoku.prototype.clear = function() {
    this.request("write.cgi?clear");
}

function ryoku_clear() {
    ryoku.clear();
    return false;
}
