#!/usr/bin/env ruby

require 'pstore'
require 'mkgif'
require 'util'

q = ENV['QUERY_STRING']

if q.empty?
  require 'loadimg'
  loadimg
  exit(0)
end

print %Q(Content-Type: text/plain
Cache-Control: no-cache
Pragma: no-cache

)

def idgen(prev)
  now = Time.now.strftime('%y%m%d%H%M%S')
  cnt = if prev && now == prev[0...12]
          c = prev[12..-1].to_i + 1
          c < 10 ? "0#{c}" : c.to_s
        else
          '00'
        end
  now+cnt
end

def notice_udb(user, udb)
  now = Time.now
  udb.roots.each do |u|
    next if u == user
    pid = udb[u][:pid]
    if pid
      begin
        Process.kill(:USR1, pid)
      rescue
      end
      udb[u].delete(:pid)
    else
      udb[u][:up] = true
    end
    if !udb[u].key?(:last) || (now - udb[u][:last]) > 300
      udb.delete(u)
    end
  end
end

def notice
  udb = get_udb
  udb.transaction do
    notice_udb(query, udb)
  end
end

db = get_db

if q == 'undo'
  udb = get_udb
  count = nil
  points = []
  user = query
  udb.transaction do
    if !udb.root?(user) || !udb[user][:lastp]
      require 'loadimg'
      loadimg_body
      exit(0)
    end
    points = udb[user][:lastp]
    udb[user][:lastp] = nil
    db.transaction do
      if db.root?('points')
        pp = db['points'].join(',') + ','
        rp = points.join(',') + ','
        points = (db['points'] = pp.sub(rp, '').chop.split(',').map{|i|i.to_i})
        db['count'] = count = idgen(db['count'])
      end
    end
  end

  gifname = "tmp/bg#{count}.gif"
  mkgif(points, gifname)
  File.chmod(0644, gifname)

  print "#{gifname}"

  udb.transaction do
    notice_udb(user, udb)
  end

  exit(0)
elsif q == 'clear'
  now = Time.now.strftime('%y%m%d%H%M%S')
  points = nil
  db.transaction do
    if db.root?('points')
      points = db['points']
      db['points'] = []
      db['count'] = nil
    end
  end
  if points && points.size > 10
    gifname = "log/ryoku#{now}.gif"
    mkgif(points, gifname)
    File.chmod(0644, gifname)

    Dir.glob('tmp/bg?*.gif') do |f|
      File.unlink(f)
    end

    print "tmp/bg.gif"
    notice()
    exit(0)
  end
end

points = []
curpoints = []

if q.size > 0
  curpoints = q.chop.split(',').map{|i| i.to_i}
end

count = nil
db.transaction do
  if !db.root?('points')
    db['points'] = []
  end
  db['count'] = count = idgen(db['count'])
  points = db['points'] + curpoints
  db['points'] = points
end

gifname = "tmp/bg#{count}.gif"
mkgif(points, gifname)
File.chmod(0644, gifname)

udb = get_udb
udb.transaction do
  user = query
  if !udb.root?(user)
    udb[user] = {}
  end
  udb[user][:lastp] = curpoints

  notice_udb(user, udb)
end

print "#{gifname}"
